# Automatically generated by pb2py
from .. import protobuf as p


class ButtonRequest(p.MessageType):
    MESSAGE_WIRE_TYPE = 26
    FIELDS = {
        1: ('code', p.UVarintType, 0),
        2: ('data', p.UnicodeType, 0),
    }

    def __init__(
        self,
        code: int = None,
        data: str = None
    ) -> None:
        self.code = code
        self.data = data
