# Automatically generated by pb2py
from .. import protobuf as p


class EthereumTxAck(p.MessageType):
    MESSAGE_WIRE_TYPE = 60
    FIELDS = {
        1: ('data_chunk', p.BytesType, 0),
    }

    def __init__(
        self,
        data_chunk: bytes = None
    ) -> None:
        self.data_chunk = data_chunk
