syntax = "proto2";

message CoinDef {
	optional string coin_name = 1;
	optional string coin_shortcut = 2;
	optional string coin_label = 3;
	optional string curve_name = 4;
	optional uint32 address_type = 5;
	optional uint32 address_type_p2sh = 6;
	optional uint64 maxfee_kb = 7;
	optional uint64 minfee_kb = 8;
	optional bytes signed_message_header = 9;
	optional bytes hash_genesis_block = 10;
	optional uint32 xprv_magic = 11;
	optional uint32 xpub_magic = 12;
	optional uint32 xpub_magic_segwit_p2sh = 13;
	optional uint32 xpub_magic_segwit_native = 14;
	optional string bech32_prefix = 15;
	optional string cashaddr_prefix = 16;
	optional uint32 slip44 = 17;
	optional bool segwit = 18;
	optional bool decred = 19;
	optional uint32 fork_id = 20;
	optional bool force_bip143 = 21;
	optional uint64 dust_limit = 22;
	optional string address_prefix = 23;
	optional uint32 min_address_length = 24;
	optional uint32 max_address_length = 25;
	optional bytes icon = 26;
	optional uint32 version_group_id = 27;
	optional string website = 28;
	optional string github = 29;
	optional string maintainer = 30;
	optional uint32 blocktime_seconds = 31;
}
