# Automatically generated by pb2py
from .. import protobuf as p


class Success(p.MessageType):
    MESSAGE_WIRE_TYPE = 2
    FIELDS = {
        1: ('message', p.UnicodeType, 0),
    }

    def __init__(
        self,
        message: str = None
    ) -> None:
        self.message = message
