# Automatically generated by pb2py
from .. import protobuf as p


class CosiCommitment(p.MessageType):
    MESSAGE_WIRE_TYPE = 72
    FIELDS = {
        1: ('commitment', p.BytesType, 0),
        2: ('pubkey', p.BytesType, 0),
    }

    def __init__(
        self,
        commitment: bytes = None,
        pubkey: bytes = None
    ) -> None:
        self.commitment = commitment
        self.pubkey = pubkey
