# Automatically generated by pb2py
from .. import protobuf as p
if __debug__:
    try:
        from typing import List
    except ImportError:
        List = None


class CosiSign(p.MessageType):
    MESSAGE_WIRE_TYPE = 73
    FIELDS = {
        1: ('address_n', p.UVarintType, p.FLAG_REPEATED),
        2: ('data', p.BytesType, 0),
        3: ('global_commitment', p.BytesType, 0),
        4: ('global_pubkey', p.BytesType, 0),
    }

    def __init__(
        self,
        address_n: List[int] = None,
        data: bytes = None,
        global_commitment: bytes = None,
        global_pubkey: bytes = None
    ) -> None:
        self.address_n = address_n if address_n is not None else []
        self.data = data
        self.global_commitment = global_commitment
        self.global_pubkey = global_pubkey
